#!/usr/bin/env node
"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var commander_1 = __importDefault(require("commander"));
var __1 = __importDefault(require(".."));
var helpers_1 = require("../helpers");
var manifest = JSON.parse(fs_1.default.readFileSync(path_1.default.resolve(__dirname, '../../package.json'), 'utf8'));
var NODE_ARGS = [
    ['debug-port', 'debugPort', true],
    ['inspect-port', 'inspectPort', true],
    ['inspect', 'inspect', false],
    ['inspect-brk', 'inspectBrk', false],
    ['inspect-publish-uid', 'inspectPublishUid', true],
    ['enable-source-maps', 'enableSourceMaps', false],
];
commander_1.default
    .version(manifest.version)
    .description(manifest.description)
    .usage('[options] <source directory>')
    .option('-w, --watch', 'Watch files for changes')
    .option('--root <directory>', 'Root directory for compilation; where presets and CLI config is resolved from (defaults to cwd)')
    .option('--ignored <glob>', 'Ignored files and directories that match the given glob')
    .option('--ignored-for-restart <glob>', 'These files are transpiled, but do not cause restart')
    .option('--source-maps [true|false|inline]', 'Generate source maps for transpiled files', function (value) {
    return value !== 'inline' ? value === 'true' : 'inline';
})
    .option('--reset-cache', 'Retranspile all files ignoring cache')
    .option('--keep-extra-files', 'Do NOT delete extra files in the output directory')
    .option('-o, --output-directory <directory>', 'Output directory to write transpiled files to')
    .option('-x, --execute <entryFile>', 'Relative path of file to execute (only supported in watcher mode)')
    .option('--execute-delay <delay>', 'Delay in ms in between restarts of executed file (defaults to 1000ms)', function (value) { return parseInt(value, 10) || 1000; })
    .option('--extensions <exts>', 'Comma spearated extensions to process through the CLI (defaults to .js)', function (value) {
    return value
        .split(',')
        .map(function (item) { return item.trim(); })
        .filter(Boolean);
})
    .option('--no-load-config', 'Disables loading of "sb-config-file" from package.json (in --root)')
    .option('--print-config', 'Print the config being used (for debugging only)')
    .on('--help', function () {
    console.log('\nArguments after -- will be passed as-are to programs executed through -x');
})
    .combineFlagAndOptionalValue(true);
NODE_ARGS.forEach(function (_a) {
    var _b = __read(_a, 3), nodeArg = _b[0], hasOption = _b[2];
    commander_1.default.option("--" + nodeArg + (hasOption ? ' <arg>' : ''), 'Passthrough arg for Node.js runtime for programs executed through -x');
});
commander_1.default.parse(process.argv);
if (commander_1.default.args.length < 1) {
    commander_1.default.outputHelp();
    process.exit(1);
}
var nodeArgs = NODE_ARGS.filter(function (item) { return commander_1.default[item[1]] != null; })
    .map(function (item) { return (item[2] ? ["--" + item[0], commander_1.default[item[1]]] : ["--" + item[0]]); })
    .flat();
var specifiedArgs = [];
function optionalGet(obj, key, defaultValue) {
    if (obj[key] == null) {
        return defaultValue;
    }
    specifiedArgs.push(key);
    return obj[key];
}
var configPartial = {
    sourceDirectory: commander_1.default.args[0],
    outputDirectory: optionalGet(commander_1.default, 'outputDirectory', ''),
    rootDirectory: optionalGet(commander_1.default, 'root', process.cwd()),
    watch: optionalGet(commander_1.default, 'watch', false),
    ignored: optionalGet(commander_1.default, 'ignored', ''),
    ignoredForRestart: optionalGet(commander_1.default, 'ignoredForRestart', ''),
    sourceMaps: optionalGet(commander_1.default, 'sourceMaps', false),
    resetCache: optionalGet(commander_1.default, 'resetCache', false),
    keepExtraFiles: optionalGet(commander_1.default, 'keepExtraFiles', false),
    execute: optionalGet(commander_1.default, 'execute', ''),
    executeDelay: optionalGet(commander_1.default, 'executeDelay', 250),
    extensions: optionalGet(commander_1.default, 'extensions', ['.js']),
    loadConfig: optionalGet(commander_1.default, 'loadConfig', true),
    printConfig: optionalGet(commander_1.default, 'printConfig', false),
    nodeArgs: nodeArgs,
    programArgs: commander_1.default.args.slice(1),
};
__1.default(__assign(__assign({}, configPartial), { specifiedArgs: specifiedArgs })).catch(function (error) {
    helpers_1.logError(error);
    process.exit(1);
});
